"""
MolPortWebServices.py - library for accessing MolPort web services

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2016
stardrop-support@optibrium.com
"""

import urllib
import urllib2
import json
import sys


BASE_URL = "https://api.molport.com/api/"


class SearchTypes:
    """
    An "enum" class which defines several enumeration values and some static
    helper functions for use with this type
    """
    Substructure, Superstructure, Exact, Similarity, Perfect, \
        ExactFragment = range(1, 7)

    def __init__(self):
        """
        Defined for PEP8, this class should not be instantiated at any point
        """
        raise RuntimeError("Enum class SearchTypes instantiated")

    @staticmethod
    def is_search_type(val):
        """
        This function determines whether a particular value is compatible with a
        SearchTypes value.

        Usage:
        >>> val1 = 1 # Equivalent to SearchTypes.Substructure
        >>> valid = SearchTypes.is_search_type(val1)
        >>> print valid # True
        >>> val3 = 9
        >>> valid = SearchTypes.is_search_type(val3)
        >>> print valid # False
        >>> val4 = list()
        >>> valid = SearchTypes.is_search_type(val4)
        >>> print valid # False

        :param val: The value to test to see if it is compatible with
                    SearchTypes
        :return: True if the value is compatible, false otherwise
        :rtype: Boolean
        """
        if val in range(1, 7):
            return True
        else:
            return False


class MolPortWebServices:
    def __init__(self, username, authentication_code):
        """
        Create a new instance of MolPortWebServices with the provided
        credentials

        :param username:            The username to use while accessing the API
        :param authentication_code: The password to use while accessing the API
        """
        self.username = username
        self.authentication_code = authentication_code

    def load_molecule(self, molecule_id):
        """
        Load a molecule with the provided MolPort molecule ID

        (API method "Load molecule" - 3.1)

        :param molecule_id: MolPort molecule ID e.g.
                            MolPort compound ID 2325020 / MolPort-002-325-020
        :return: Detailed information about the specified molecule
        """
        if not self.authentication_code:
            return None

        url = BASE_URL + "molecule/load?"
        arguments = {"molecule": molecule_id,
                     "username": self.username,
                     "authenticationcode": self.authentication_code}
        url += urllib.urlencode(arguments)

        resolver = urllib2.urlopen(url)
        json_data = resolver.read()

        result = None
        try:
            result = json.loads(json_data)
        except ValueError:
            print result

        return result

    def chemical_search(self,
                        smiles,
                        search_type,
                        max_search_time,
                        max_result_count,
                        similarity_index):
        """
        Perform a chemical search based on the provided input SMILES using the
        specified search strategy. The input may be a full structure or a
        fragment based on the particular search type specified.

        :param smiles:              The structure/fragment to search for
        :param search_type:         The chemical search type to use
        :param max_search_time:     Maximum time in milliseconds the search may
                                    run
        :param max_result_count:    Maximum number of results, capped at 1000
                                    (default 1000)
        :param similarity_index:    If using a similarity search then this is
                                    used to specify similarity in the range 0-1
        :type smiles: str
        :type search_type: int
        :type max_search_time: int
        :type max_result_count: int
        :type similarity_index: float
        :return:
        """
        if not self.authentication_code:
            return None
        
        result = None
        try:
            request_data = {
                "User Name": self.username,
                "Authentication Code": self.authentication_code,
                "Structure": smiles,
                "Search Type": search_type,
                "Maximum Search Time": max_search_time,
                "Maximum Result Count": max_result_count,
                "Chemical Similarity Index": similarity_index
            }
            request = json.dumps(request_data)

            post_data_map = {"json": request}
            post_data = urllib.urlencode(post_data_map)

            url = BASE_URL + "chemical-search/search"
            json_data = None
            try:
                resolver = urllib2.urlopen(url, post_data)
                json_data = resolver.read()
            except URLError as err:
                print 'Error resolving MolPort structure query: ', err.strerror
            except:
                print "Unexpected error:", sys.exc_info()[0]

            try:
                result = json.loads(json_data)
            except ValueError:
                print result
        except:
            print "Unexpected error:", sys.exc_info()[0]

        return result
